<?php

namespace Teknisa\CMP\Util;

use Teknisa\Libs\Util\SMTP;
use Teknisa\Libs\Util\Utilities;
use Zeedhi\Framework\DependencyInjection\InstanceManager;

class CmpUtilities
{
    protected $connection;
    protected $newCode;
    protected $newCodeSC;
    protected $fileServerURL;
    protected $crudService;
    private $isUsingCustomSmtp;
    protected $libUtilities;

    public function __construct($connection, $newCode, $newCodeSC, $environment, $fileServerURL, $libUtilities, $crudService)
    {
        $this->connection    = $connection;
        $this->newCode       = $newCode;
        $this->newCodeSC     = $newCodeSC;
        $this->environment   = $environment;
        $this->fileServerURL = $fileServerURL;
        $this->libUtilities  = $libUtilities;
        $this->crudService   = $crudService;
    }
    
    public function generateCode($tableName)
    {
        $newCode = $this->newCode->getCodeByTableName($tableName, $this->environment->getNrOrgTrab());
        
        return $newCode;
    }

    /**
     * Execute a newCodeSC on a table name received.
     * 
     * @param string $tableName
     */
    public function generateCodeSC($tableName)
    {
        $newCode = $this->newCodeSC->getCodeByTableName($tableName, $this->environment->getNrOrgTrab());
        
        return $newCode;
    }
    
    /**
     * Define Cookie 
     */
    public function defineCookieEmail($name, $value = "", $time = 0, $path = "/", $domain = "", $secure = false, $httponly = false)
    {
        setcookie($name, $value, $time, $path, $domain, $secure, $httponly);
    }
    
    /**
     * Retorna um Cookie pelo nome
     */
    public function retornaCookieEmail($name)
    {
        if (array_key_exists($name, $_COOKIE)) {
            return $_COOKIE[$name];
        }
        
        return false;
    }
    
    public function retornarEmailsRelacionadosFornecedor(array $fornecedores)
    {
        $listaEmails = array();
        
        $fornecedores = array_unique($fornecedores);
        
        foreach ($fornecedores as $fornecedor) {
            $emailFornecedores = $this->emailsRelacionadosFornecedor($fornecedor)->getRows()[0];
        
            if ($emailFornecedores['DSLOGINECO']) {
                $listaEmails = array_merge($listaEmails, explode(',', $emailFornecedores['DSLOGINECO']));
                
                if ($emailFornecedores['DSEMAILECO']) {
                    $listaEmails = array_merge($listaEmails, explode(',', $emailFornecedores['DSEMAILECO']));
                }
            } 
        }
        
        if (!count($listaEmails)) {
            throw new \Exception("Expiração alterada. <br/> E-mail não enviado pois o fornecedor não possui acesso as cotações.");
        }
        
        $listaEmails = array_unique($listaEmails);
        
        return $listaEmails;
    }
    
    public function emailsRelacionadosFornecedor($cdforneced)
    {
        $conditions = [
            array(
                "columnName" => "CDFORNECED",
                "operator"   => '=',
                "value"      => $cdforneced
            )];
        
        return $this->crudService->getFilteredDataSet('emailsRelacionadosFornecedor', $conditions);
    }
    
    public function fileGetContentUtf8($fileName)
    {
        $content = file_get_contents($fileName);
        
        return mb_convert_encoding($content, 'UTF-8', mb_detect_encoding($content, 'UTF-8, ISO-8859-1', true));
    }
    
    public function returnLogo($idAplicativo)
    {
        $sql = "SELECT URLLOGO FROM APPCONFIG WHERE IDAPLICATIVO = :IDAPLICATIVO AND NRORG = :P_NRORG";
        $stmt = $this->connection->prepare($sql);
        $stmt->execute(
            array(
                "IDAPLICATIVO" => $idAplicativo,
                "P_NRORG" => $this->environment->getNrorgTrab()
            )
        );
        
		$result = $stmt->fetchAll();
		
		if (isset($result[0]) && $result[0]['URLLOGO'] && $result[0]['URLLOGO'] != null && $result[0]['URLLOGO'] != '') {
    		return $this->fileServerURL.$result[0]['URLLOGO'];
        } else {
            return false;
        }
    }
    
    public function returnLgpd($cdFilial)
    {
        $sql = "SELECT P.TXAVISOLEGALEMAIL FROM PARASUPRI P, FILIAL F WHERE F.CDFILIAL = :P_CDFILIAL AND P.CDFILIAL = NVL(F.CDFILCENTCMP, F.CDFILIAL)";
        $stmt = $this->connection->prepare($sql);
        $stmt->execute(
            array(
                "P_CDFILIAL" => $cdFilial
            )
        );
        
		$result = $stmt->fetchAll();
		
        return $result[0]['TXAVISOLEGALEMAIL'];
    }
    
    public function messageSendMail($cdFilial = null, $assunto, $message, $idAplicativo = 'PORTAL E-COMMERCE', $attrImg = 'width="175px"')
    {
        if(is_null($cdFilial)){
            $cdFilial = $this->environment->getCdFilial();
        }
        $logoSrc = $this->returnLogo($idAplicativo);
        $this->getLogoAttributes($logoSrc, $attrImg);
        $lgpd = $this->returnLgpd($cdFilial);
        $messageSendMail = $this->returnMessageSendMail($logoSrc, $assunto, $message, $attrImg, $lgpd);
        
        return $messageSendMail;
    }
    
    public function checkFileExists($url)
    {
        $ch = curl_init($url);
        curl_setopt($ch, CURLOPT_NOBODY, true);
        curl_exec($ch);
        $code = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        
        if ($code == 200) {
            $status = true;
        } else {
            $status = false;
        }
        
        curl_close($ch);
        
        return $status;
    }
    
    public function getLogoAttributes(&$logoSrc, &$attrImg)
    {
        if ($logoSrc != false && $this->checkFileExists($logoSrc)) {
            $imgAttributes = getimagesize($logoSrc);
            
            if ($imgAttributes[1] > $imgAttributes[0]) {
                $attrImg = 'height="175px"';
            }
        } else {
            $logoSrc = 'https://www.teknisa.com/imagens/logo-teknisa.png';
        }
    }
    
    public function returnMessageSendMail($logoSrc, $assunto, $message, $attrImg, $lgpd)
    {
        $templatesDir = realpath(__DIR__ . '/Template');
        $templateEmail = $this->fileGetContentUtf8($templatesDir . "/email.html");
        $replace = array(
                $attrImg,
                $logoSrc,
                $assunto,
                $message,
                $lgpd
        );
            
        $varReplaced = array(
            '%attrImg%',
            '%logoSrc%',
            '%assunto%',
            '%mensagem%',
            '%lgpd%'
        );
        
        return $messageSendMail = str_replace($varReplaced, $replace, $templateEmail);
    }
    
    public function enviaEmail($fromName, $subject, $address, $messageSendMail, $emailDestino, $emailCopia = false, $attachments = false, $clearAttachments = false)
    {
        $this->useCustomSmtp();
        
        if (!$this->isUsingCustomSmtp && $fromName != '') {
            $this->smtpUtil->setFromName($fromName);
            $this->smtpUtil->setNameReplyTo($fromName);
        }

        if (!empty($attachments)) {
            $this->addAttachments($attachments, $clearAttachments);
        }

        if($address != '') {
            $this->smtpUtil->setNmAddress($address);
        }
        $this->smtpUtil->setSubject($subject);
        
        if ($emailCopia) {
            return $this->smtpUtil->sendMail($messageSendMail, $emailDestino, $emailCopia);
        } else {
            return $this->smtpUtil->sendMail($messageSendMail, $emailDestino);
        }
    }

    private function addAttachments($attachments, $clearAttachments = false )
    {
        if ($clearAttachments) {
            $this->smtpUtil->clearAttachments();
        }

        foreach ($attachments as $attachment) {
            $this->smtpUtil->addAttachment(
                $attachment->getFile(),
                $attachment->getName(),
                $attachment->getEncoding(),
                $attachment->getType(),
                $attachment->getDisposition(),
                $attachment->isStringAttachment()
            );
        }
    }

    public function useCustomSmtp()
    {
        $params = $this->getCustomSmtpSettings();

        $this->smtpUtil = new SMTP($params);

        if (isset($params['fromName'])) {
            $this->smtpUtil->setFromName($params['fromName']);
        }

        if (isset($params['from'])) {
            $this->smtpUtil->setNameReplyTo($params['from']);
        }

        $this->isUsingCustomSmtp = true;
    }

    public function getCustomSmtpSettings()
    {
        $tipoDadoUtilizado = $this->getTipoDadoUilizadoEnvioEmail()['IDUTDADOPERENVEMAIL'];
        $dadosSMTPFilial   = $this->getDadosSMTPFromFilial();
        $settings          = self::getSmtpSettingsFromAddress($dadosSMTPFilial['DSSMTPEMAILF']);
        
        
        $params['host'] = $settings['host'];
        
        if (isset($settings['port'])) {
            $params['port'] = $settings['port'];
        }

        switch($tipoDadoUtilizado) {
            case 'LOGIN_FILIAL_REMETENTE_FILIAL':
                $params['from']     = $dadosSMTPFilial['DSEMAILFILI'];
                $params['fromName'] = $dadosSMTPFilial['DSEMAILFILI'];
                $params['username'] = $dadosSMTPFilial['DSLOGINSMTP'];
                $params['password'] = $dadosSMTPFilial['CDSENHEMAILF'];
                break;
            case 'LOGIN_FILIAL_REMETENTE_OPER':
                $dadosEmailOperadorLogado = $this->getDadosEmailOperadorLogado();
                
                $params['from']     = $dadosEmailOperadorLogado['DSEMAILOPER'];
                $params['fromName'] = $dadosEmailOperadorLogado['NMOPERADOR'];
                $params['username'] = $dadosSMTPFilial['DSLOGINSMTP'];
                $params['password'] = $dadosSMTPFilial['CDSENHEMAILF'];
                break;
            case 'LOGIN_OPER_REMETENTE_OPER':
                $dadosEmailOperadorLogado = $this->getDadosEmailOperadorLogado();

                $params['from']     = $dadosEmailOperadorLogado['DSEMAILOPER'];
                $params['fromName'] = $dadosEmailOperadorLogado['NMOPERADOR'];
                $params['username'] = $dadosEmailOperadorLogado['DSEMAILOPER'];
                $params['password'] = $dadosEmailOperadorLogado['CDSENEMAILOP'];
                break;
        }
        
        $params['password'] = $this->getPassword($params, $tipoDadoUtilizado);
        
        $securityParameters = $this->getSecurityParameters($dadosSMTPFilial, $params);
        $params = array_merge($params, $securityParameters);
        return $params;
    }
    
    /**
     * Recupera a senha descriptografada. Caso a senha não exista nos parâmetros,
     * retorna uma string vazia.
     * 
     * @param array $customSmtpSettings
     * 
     * @return string Contendo a senha devidamente tratada.
     */
    private function getPassword($customSmtpSettings, $tipoDadoUtilizado)
    {
        if (!isset($customSmtpSettings['password']) || empty($customSmtpSettings['password'])) {
            return '';
        }
        
        if ($tipoDadoUtilizado == 'LOGIN_OPER_REMETENTE_OPER') {
            return $this->libUtilities->getDelphiPasswordEncrypted($customSmtpSettings['password'], false);   
        }
        
        return self::getDelphiPasswordEncrypted($customSmtpSettings['password'], false);
    }
    
    /**
     * Recupera informações de segurança para usar na conexão com o SMTP.
     * Utiliza dos parâmetros recuperados para definir as seguintes propriedades:
     * . smtpSecure:
     *      - 'tls', caso o parâmetro IDUTTLSEMAIL esteja marcado;
     *      - 'ssl', caso o parâmetro IDUTSSLEMAIL esteja marcado;
     *      - '', caso ambos os parâmetros acima estejam desmarcados;
     * . smtpAuth:
     *      - true, caso o usuário e senha estejam preenchidos;
     *      - false, caso usuário ou senha estejam em branco;
     * 
     * @param array $dadosSMTPFilial
     * @param array $customSmtpSettings
     * 
     * @return array Contendo os parâmetros de segurança.
     */
    private function getSecurityParameters($dadosSMTPFilial, $customSmtpSettings)
    {
        $params = [
            'smtpSecure' => '',
            'smtpAuth' => false
        ];
        
        if ($dadosSMTPFilial['IDUTTLSEMAIL'] == 'S') {
            $params['smtpSecure'] = 'tls';
        } else if ($dadosSMTPFilial['IDUTSSLEMAIL'] == 'S') {
            $params['smtpSecure'] = 'ssl';
        }
        
        if (!empty($customSmtpSettings['username']) && !empty($customSmtpSettings['password'])) {
            $params['smtpAuth'] = true;
        }
        
        return $params;
    }
    
    /**
     * Recupera informações do endereço de SMTP, como:
     * - host;
     * - porta;
     * 
     * @param string $url Url do servidor com ou sem porta.
     * 
     * @return array Contendo as informações encontradas.
     */
    public function getSmtpSettingsFromAddress($url)
    {
        $match = self::splitByCommonDelimiters($url);
        
        if (empty($match)) {
            throw new \Exception('Formato do endereço de SMTP inválido.');
        }
        
        if (count($match) > 1) {
            $params['port'] = $match[1];
        }
        
        $params['host'] = $match[0];
        
        return $params;
    }
        
    public function splitByCommonDelimiters($string)
    {
        $pattern = '/[:;|]+/';
        $cleanedString = self::removeAllWhitespaces($string);
        
        $splittedString = preg_split($pattern, $cleanedString);
        
        return $splittedString;
    }
    
    public function removeAllWhitespaces($string)
    {
        $pattern = '/\s+/';
        
        $cleanedString = preg_replace($pattern, '', $string);
        
        return $cleanedString;
    }

    public function getTipoDadoUilizadoEnvioEmail()
    {
        $query = "SELECT NVL(PARASUPRI.IDUTDADOPERENVEMAIL, 'LOGIN_FILIAL_REMETENTE_FILIAL') IDUTDADOPERENVEMAIL
                        FROM PARASUPRI
                    WHERE PARASUPRI.CDFILIAL = :P_CDFILCENTCMP";
        $std = $this->connection->prepare($query);
        
        $std->execute(array(
            "P_CDFILCENTCMP" => self::getCentralCompras($this->environment->getCdFilial())['CDFILCENTCMP'],
        ));
        
        return $std->fetch();
    }
    
    
    /**
     * Método que retorna a central de compras da filial informada.
     * Caso o código de filial não seja informado, recebe null
     * como default e atribui a veriável o valor da filial logada
     * 
     * @param {string} codigoFilial código da filial desejada ou
     * caso não informado, da filial logada.
     * 
     * @return {object} retorna uma unica row contendo as informações da
     * unidade matriz.
    */
    public function getCentralCompras($codigoFilial = null)
    {
        if (!$codigoFilial) {
            $codigoFilial = $this->environment->getCdFilial();
        }
        
        $query = "SELECT
                        F.CDFILIAL,
                        F.NMFILIAL,
                        F2.CDFILIAL   AS CDFILCENTCMP,
                        F2.NMFILIAL   AS NMFILCENTCMP,
                        F2.NRORG
                    FROM
                        FILIAL   F,
                        FILIAL   F2
                    WHERE
                        F.CDFILIAL = :CDFILIAL
                        AND F2.CDFILIAL = NVL(F.CDFILCENTCMP, F.CDFILIAL)";
        $std = $this->connection->prepare($query);
        
        $std->execute(array(
            "CDFILIAL" => $codigoFilial,
        ));
        return $std->fetch();
    }

    public function getDadosSMTPFromFilial()
    {
        $query = "SELECT EF.CDSENHEMAILF, EF.DSSMTPEMAILF, NVL(EF.DSLOGINSMTP, EF.DSEMAILFILI) DSLOGINSMTP,
                          NVL(EF.DSEMAILFILI, EF.DSLOGINSMTP) DSEMAILFILI, EF.IDUTTLSEMAIL,
                           EF.IDUTSSLEMAIL, FI.CDFILIAL||' - '||FI.NMFILIAL AS NMFILIAL
                      FROM ENDEFILI EF INNER JOIN FILIAL FI ON EF.CDFILIAL = FI.CDFILIAL
                     WHERE EF.CDFILIAL     = :P_CDFILIPROC
                       AND EF.IDTPENDEFILI = 'P'";
        $std = $this->connection->prepare($query);
        
        $std->execute(array(
            "P_CDFILIPROC" => $this->environment->getCdFilial(),
        ));
        
        return $std->fetch();
    }
    
    public function getDadosEmailOperadorLogado()
    {
        $query = "SELECT OP.NMOPERADOR, OP.DSEMAILOPER, OP.CDSENEMAILOP
                        FROM OPERADOR OP
                    WHERE OP.CDOPERADOR = :P_CDOPERADOR";
        $std = $this->connection->prepare($query);
        
        $std->execute(array(
            "P_CDOPERADOR" => $this->environment->getCdOperador(),
        ));
        
        return $std->fetch();
    }

    public function getVisaoSistema()
    {
        $filialLogada = $this->environment->getCdFilial();
        $centralCompras = $this->getCentralCompras($filialLogada)['CDFILCENTCMP'];

        if ($centralCompras == $filialLogada) {
            return 'SUP';
        }

        return 'CMP';
    }
    
    /**
     * Temporário até que uma solução seja implementada no local original desta
     * função.
     * 
     * A original realiza chamada apenas nas funções GetCriptoPass e GetUnCriptoPass,
     * equivalentes ao CriptoPlus e UncriptoPlus do Delphi.
     * 
     * Porém, utilizamos as criptografias convencionais, que até então não eram
     * implementadas la Bib_html.
     * 
     * Foi incluída na 'Bib', rdm 'rdmDBGeral', na issue #238324
     * 
     * Teknisa\Libs\Util\Utilities
     */
    public static function getDelphiPasswordEncrypted($password, $isEncrypt = true, $offset = '2')
    {
        $url = self::getAspUrlParameter() . '/comwebservice/service.asmx?wsdl';
        
        $options = array(
            'encoding'    => 'ISO-8859-1',
            'trace'       => true,
            'exceptions'  => true,
            'compression' => SOAP_COMPRESSION_ACCEPT | SOAP_COMPRESSION_GZIP,
        );
        

        $args = array(
            new \SoapVar($password, XSD_STRING, 'string', "http://www.w3.org/2001/XMLSchema"),
            new \SoapVar($offset, XSD_STRING, 'string', "http://www.w3.org/2001/XMLSchema"),
        );

        $cryptType = $isEncrypt ? 'GetCripto' : 'GetUncripto';
        
        $params = array(
            'rdm'    => 'Bib.rdmDBGeral',
            'method' => $cryptType,
            'args'   => $args,
        );

        try {
            $client = new \SoapClient($url, $options);
            $result = $client->executeCOMMethod($params);
            $encryption = $result->executeCOMMethodResult;
        } catch (\SoapFault $e) {
            throw new \Exception($e->getMessage());
        }

        return $encryption;
    }
    
    /**
     * Temporário até que uma solução seja implementada para a função getDelphiPasswordEncrypted.
     * 
     * Teknisa\Libs\Util\Utilities 
     */
    public static function getAspUrlParameter() {
        return InstanceManager::getInstance()->getParameter('aspUrl');
    }
    
    /**
     * Transforma os parâmetros de um request do tipo Filter (array de array)
     * em row (array chave/valor), facilitando o acesso ao valor através do nome.
     * Ex.:
     * - Antes:  $array[0]['value'] para recuperar o valor do primeiro parâmetro.
     * - Depois: $array['columnName'] para recuperar o valor do parâmetro 'columnName'.
     * 
     * @param Zeedhi\Framework\DTO\Request\Filter $request
     * 
     * @return array Contendo o nome e valor dos parâmetros encontrados.
     */
    public function getParametersFromCondititions(\Zeedhi\Framework\DTO\Request\Filter $request)
    {
        $conditions = $request->getFilterCriteria()->getConditions();
        
        return array_merge(...array_map(function($param) {
        	return [ $param['columnName'] => $param['value'] ];
        }, $conditions));
    }
}
